@@ -33,6 +33,13 @@ int      vlistInqVarMissvalUsed(int vlistID, int varID);
 #endif
 
 
+static inline int
+cdiUUIDIsNull(const unsigned char uuid[CDI_UUID_SIZE])
+{
+  static unsigned char uuid_nil[CDI_UUID_SIZE];
+  return !memcmp(uuid, uuid_nil, CDI_UUID_SIZE);
+}
+
 #include "printinfo.h"
 
 void cdiDefTableID(int tableID);
