@@ -264,6 +264,8 @@ int     cdiEncodeTime(int hour, int minute, int second);
 
 /* STREAM control routines */
 
+int     cdiGetFiletype(const char *path, int *byteorder);
+
 /*      streamOpenRead: Open a dataset for reading */
 int     streamOpenRead(const char *path);
 
@@ -318,7 +320,6 @@ int     streamInqCurTimestepID(int streamID);
 
 char   *streamFilename(int streamID);
 char   *streamFilesuffix(int filetype);
-int     streamNtsteps(int streamID);
 off_t   streamNvals(int streamID);
 
 int     streamInqNvars ( int streamID );
@@ -785,7 +786,7 @@ void    gridDefYbounds(int gridID, const double *ybounds_vec);
 /*      gridInqYbounds: Get the bounds of a Y-axis */
 int     gridInqYbounds(int gridID, double *ybounds_vec);
 
-void    gridDefRowlon(int gridID, int nrowlon, const int *rowlon_vec);
+void    gridDefRowlon(int gridID, int nrowlon, const int rowlon_vec[]);
 void    gridInqRowlon(int gridID, int *rowlon_vec);
 void    gridChangeType(int gridID, int gridtype);
 
