@@ -17,19 +17,37 @@
 #define STRING(x)	XSTRING(x)
 
 static char gribapi_libvers[64] = "";
+#if  defined  (HAVE_LIBGRIB_API)
+static int gribapi_libvers_init;
+#endif
 
-const char *gribapiLibraryVersion(void)
+
+void gribapiLibraryVersion(int* major_version, int* minor_version, int* revision_version)
 {
 #if  defined  (HAVE_LIBGRIB_API)
   long version = grib_get_api_version();
-  int major_version, minor_version, revision_version;
+  (*major_version)    = version/10000;
+  (*minor_version)    = (version-(*major_version)*10000)/100;
+  (*revision_version) = (version-(*major_version)*10000-(*minor_version)*100);
+#else
+  (*major_version)    = 0;
+  (*minor_version)    = 0;
+  (*revision_version) = 0;
+#endif
+}
 
-  major_version    = version/10000;
-  minor_version    = (version-major_version*10000)/100;
-  revision_version = (version-major_version*10000-minor_version*100);
+const char *gribapiLibraryVersionString(void)
+{
+#if  defined  (HAVE_LIBGRIB_API)
+  if (!gribapi_libvers_init)
+    {
+      int major_version, minor_version, revision_version;
 
-  sprintf(gribapi_libvers, "%d.%d.%d",
-	  major_version, minor_version, revision_version);
+      gribapiLibraryVersion(&major_version, &minor_version, &revision_version);
+
+      sprintf(gribapi_libvers, "%d.%d.%d", major_version, minor_version, revision_version);
+      gribapi_libvers_init = 1;
+    }
 #endif
 
   return (gribapi_libvers);
@@ -41,7 +59,7 @@ void gribContainersNew(stream_t * streamptr)
   int editionNumber = 2;
 
   if ( streamptr->filetype == FILETYPE_GRB ) editionNumber = 1;
-
+  (void)editionNumber;
 #if  defined  (HAVE_LIBCGRIBEX)
   if ( streamptr->filetype == FILETYPE_GRB )
     {
@@ -69,8 +87,8 @@ void gribContainersNew(stream_t * streamptr)
 
       streamptr->gribContainers = (void **) gribContainers;
 #else
-      gribContainer_t *gribContainers;
-      gribContainers = (gribContainer_t *) malloc(nvars*sizeof(gribContainer_t));
+      gribContainer_t *gribContainers
+        = (gribContainer_t *)xmalloc((size_t)nvars*sizeof(gribContainer_t));
 
       for ( int varID = 0; varID < nvars; ++varID )
         {
