@@ -85,8 +85,8 @@ const double *gridInqXboundsPtr(int gridID);
 const double *gridInqYboundsPtr(int gridID);
 const double *gridInqAreaPtr(int gridID);
 
-int gridCompare(int gridID, grid_t grid);
-int gridGenerate(grid_t grid);
+int gridCompare(int gridID, const grid_t *grid);
+int gridGenerate(const grid_t *grid);
 
 void gridGetIndexList( int, int * );
 
