@@ -2,8 +2,6 @@
 #  include "config.h"
 #endif
 
-#ifdef USE_MPI
-
 #include <inttypes.h>
 
 #include <mpi.h>
@@ -35,7 +33,7 @@ static struct
   { DATATYPE_UCHAR, MPI_UNSIGNED_CHAR }
 };
 
-static inline int
+static inline size_t
 lookupDt(int datatype)
 {
   for (size_t i = 0; i < sizeof (dtDict) / sizeof (dtDict[0]); ++i)
@@ -48,7 +46,7 @@ lookupDt(int datatype)
 static int dtDictMatchComplete = 0;
 
 static inline void
-  dtDictFixMPIType(int i, int typeclass, int size)
+dtDictFixMPIType(size_t i, int typeclass, int size)
 {
   MPI_Aint lb, extent;
   xmpi(MPI_Type_get_extent(dtDict[i].mpidt, &lb, &extent));
@@ -72,7 +70,8 @@ setupDtDict()
 }
 #endif
 
-int serializeGetSizeMPI(int count, int datatype, void *context)
+static int
+serializeGetSizeMPI(int count, int datatype, void *context)
 {
   int size;
   xmpi(MPI_Pack_size(count, dtDict[lookupDt(datatype)].mpidt,
@@ -81,21 +80,23 @@ int serializeGetSizeMPI(int count, int datatype, void *context)
 }
 
 
-void serializePackMPI(void *data, int count, int datatype,
-                      void *buf, int buf_size, int *position, void *context)
+static void
+serializePackMPI(void *data, int count, int datatype,
+                 void *buf, int buf_size, int *position, void *context)
 {
   xmpi(MPI_Pack(data, count, dtDict[lookupDt(datatype)].mpidt,
                 buf, buf_size, position, *(MPI_Comm *)context));
 }
 
-void serializeUnpackMPI(void *buf, int buf_size, int *position,
-                        void *data, int count, int datatype, void *context)
+static void
+serializeUnpackMPI(void *buf, int buf_size, int *position,
+                   void *data, int count, int datatype, void *context)
 {
   xmpi(MPI_Unpack(buf, buf_size, position, data, count,
                   dtDict[lookupDt(datatype)].mpidt, *(MPI_Comm *)context));
 }
 
-void serializeSetMPI()
+void cdiPioSerializeSetMPI()
 {
 #if CDI_DT_MATCH_NEEDED
   if (!dtDictMatchComplete)
@@ -110,7 +111,6 @@ void serializeSetMPI()
 }
 
 
-#endif
 /*
  * Local Variables:
  * c-file-style: "Java"
