@@ -13,10 +13,9 @@ enum
   MODEL     = 5,
   STREAM    = 6,
   VLIST     = 7,
+  RESH_DELETE,
   START     = 55555555,
   SEPARATOR = 66666666,
-  DATATOKEN = 77777777,
-  FUNCCALL  = 88888888,
   END       = 99999999
 };
 
