@@ -387,7 +387,7 @@ void cgribexAddRecord(stream_t * streamptr, int param, int *isec1, int *isec2, d
 
   cgribexGetGrid(streamptr, isec2, isec4, &grid, iret);
 
-  int gridID = varDefGrid(vlistID, grid, 0);
+  int gridID = varDefGrid(vlistID, &grid, 0);
 
   int zaxistype = grib1ltypeToZaxisType(ISEC1_LevelType);
 
@@ -854,7 +854,7 @@ int cgribexScanTimestep2(stream_t * streamptr)
   cdi_create_records(streamptr, tsID);
 
   nrecords = streamptr->tsteps[tsID].nallrecs;
-  if ( nrecords ) streamptr->tsteps[1].recIDs = (int *) malloc(nrecords*sizeof(int));
+  if ( nrecords ) streamptr->tsteps[1].recIDs = (int *)xmalloc((size_t)nrecords * sizeof(int));
   streamptr->tsteps[1].nrecs = 0;
   for ( recID = 0; recID < nrecords; recID++ )
     streamptr->tsteps[1].recIDs[recID] = -1;
