@@ -4,15 +4,14 @@
 
 #include "dmemory.h"
 #include "cdi.h"
-#include "stream_int.h"
+#include "cdi_int.h"
 #include "stream_cdf.h"
 
 
 void streamDefHistory(int streamID, int length, const char *history)
 {
-  stream_t *streamptr;
-
-  streamptr = stream_to_pointer(streamID);
+#ifdef HAVE_LIBNETCDF
+  stream_t *streamptr = stream_to_pointer(streamID);
 
   if ( streamptr->filetype == FILETYPE_NC  ||
        streamptr->filetype == FILETYPE_NC2 ||
@@ -26,21 +25,25 @@ void streamDefHistory(int streamID, int length, const char *history)
 	  len = strlen(history);
 	  if ( len )
 	    {
+              /* FIXME: what's the point of strdupx? Why not use
+               * history argument directly? */
 	      histstring = strdupx(history);
 	      cdfDefHistory(streamptr, length, histstring);
 	      free(histstring);
 	    }
 	}
     }
+#else
+  (void)streamID; (void)length; (void)history;
+#endif
 }
 
 
 int streamInqHistorySize(int streamID)
 {
   int size = 0;
-  stream_t *streamptr;
-
-  streamptr = stream_to_pointer(streamID);
+#ifdef HAVE_LIBNETCDF
+  stream_t *streamptr = stream_to_pointer(streamID);
 
   if ( streamptr->filetype == FILETYPE_NC  ||
        streamptr->filetype == FILETYPE_NC2 ||
@@ -49,16 +52,17 @@ int streamInqHistorySize(int streamID)
     {
       size = cdfInqHistorySize(streamptr);
     }
-
+#else
+  (void)streamID;
+#endif
   return (size);
 }
 
 
 void streamInqHistoryString(int streamID, char *history)
 {
-  stream_t *streamptr;
-
-  streamptr = stream_to_pointer(streamID);
+#ifdef HAVE_LIBNETCDF
+  stream_t *streamptr = stream_to_pointer(streamID);
 
   if ( streamptr->filetype == FILETYPE_NC  ||
        streamptr->filetype == FILETYPE_NC2 ||
@@ -67,6 +71,9 @@ void streamInqHistoryString(int streamID, char *history)
     {
       cdfInqHistoryString(streamptr, history);
     }
+#else
+  (void)streamID; (void)history;
+#endif
 }
 /*
  * Local Variables:
