@@ -444,8 +444,8 @@ void vlistClearFlag(int vlistID)
 }
 
 static
-int vlist_generate_zaxis(int vlistID, int zaxistype, int nlevels, double *levels,
-                         double *lbounds, double *ubounds, int vctsize, const double *vct)
+int vlist_generate_zaxis(int vlistID, int zaxistype, int nlevels, const double *levels,
+                         const double *lbounds, const double *ubounds, int vctsize, const double *vct)
 {
   int zaxisID = CDI_UNDEFID;
   int zaxisglobdefined = 0;
@@ -1106,7 +1106,7 @@ void vlistDefNtsteps(int vlistID, int nts)
     }
 }
 
-
+// This function is used in CDO!
 int vlistNtsteps(int vlistID)
 {
   vlist_t *vlistptr = vlist_to_pointer(vlistID);
