@@ -119,10 +119,9 @@ static void selEndOfPeriod(field_t *periods, field_t history, field_t current, i
     if ( DBL_IS_EQUAL(parray[i], pmissval) ) periods->nmiss++;
 }
 
-void *Consecstat (void *argument)
+void *Consecstat(void *argument)
 {
   int operatorID;
-  int i;
   int istreamID, itaxisID, ivlistID, itsID;
   int ostreamID, otaxisID, ovlistID, otsID;
   int vdate = 0, vtime = 0;
@@ -152,7 +151,7 @@ void *Consecstat (void *argument)
   vlistDefTaxis(ovlistID, otaxisID);
 
   field_init(&field);
-  field.ptr = malloc(vlistGridsizeMax(ovlistID)*sizeof(double));
+  field.ptr = (double*) malloc(vlistGridsizeMax(ovlistID)*sizeof(double));
 
   nvars     = vlistNvars(ivlistID);
   vars      = field_calloc(ivlistID, FIELD_PTR);
@@ -221,7 +220,7 @@ void *Consecstat (void *argument)
           }
 #if defined(_OPENMP)
 #pragma omp parallel for default(shared) schedule(static)
-          for ( i = 0; i < gridInqSize(vars[varID][levelID].grid); i++ )
+          for ( int i = 0; i < gridInqSize(vars[varID][levelID].grid); i++ )
             hist[varID][levelID].ptr[i] = vars[varID][levelID].ptr[i];
 #else
           memcpy(hist[varID][levelID].ptr,
