@@ -30,9 +30,6 @@
       Ensstat    enspctl         Ensemble percentiles
 */
 
-#if defined(_OPENMP)
-#  include <omp.h>
-#endif
 
 #include <cdi.h>
 #include "cdo.h"
@@ -56,7 +53,6 @@ void *Ensstat(void *argument)
   int vlistID, vlistID1, vlistID2;
   int nmiss;
   int taxisID1, taxisID2;
-  int ompthID;
   double missval;
   double *array2 = NULL;
   field_t *field;
@@ -108,15 +104,15 @@ void *Ensstat(void *argument)
       if ( !userFileOverwrite(ofilename) )
 	cdoAbort("Outputfile %s already exists!", ofilename);
 
-  ef = malloc(nfiles*sizeof(ens_file_t));
+  ef = (ens_file_t*) malloc(nfiles*sizeof(ens_file_t));
 
-  field = malloc(ompNumThreads*sizeof(field_t));
+  field = (field_t*) malloc(ompNumThreads*sizeof(field_t));
   for ( i = 0; i < ompNumThreads; i++ )
     {
       field_init(&field[i]);
       field[i].size   = nfiles;
-      field[i].ptr    = malloc(nfiles*sizeof(double));
-      field[i].weight = malloc(nfiles*sizeof(double));
+      field[i].ptr    = (double*) malloc(nfiles*sizeof(double));
+      field[i].weight = (double*) malloc(nfiles*sizeof(double));
       for ( fileID = 0; fileID < nfiles; fileID++ )
 	field[i].weight[fileID] = 1;
     }
@@ -148,9 +144,9 @@ void *Ensstat(void *argument)
   gridsize = vlistGridsizeMax(vlistID1);
 
   for ( fileID = 0; fileID < nfiles; fileID++ )
-    ef[fileID].array = malloc(gridsize*sizeof(double));
+    ef[fileID].array = (double*) malloc(gridsize*sizeof(double));
 
-  array2 = malloc(gridsize*sizeof(double));
+  array2 = (double*) malloc(gridsize*sizeof(double));
 
   tsID = 0;
   do
@@ -195,15 +191,12 @@ void *Ensstat(void *argument)
 
 	  nmiss = 0;
 #if defined(_OPENMP)
-#pragma omp parallel for default(shared) private(i, ompthID, fileID)
+#pragma omp parallel for default(shared) private(i, fileID)
 #endif
 	  for ( i = 0; i < gridsize; i++ )
 	    {
-#if defined(_OPENMP)
-	      ompthID = omp_get_thread_num();
-#else
-	      ompthID = 0;
-#endif
+	      int ompthID = cdo_omp_get_thread_num();
+
 	      field[ompthID].missval = missval;
 	      field[ompthID].nmiss = 0;
 	      for ( fileID = 0; fileID < nfiles; fileID++ )
@@ -221,7 +214,7 @@ void *Ensstat(void *argument)
 	      if ( DBL_IS_EQUAL(array2[i], field[ompthID].missval) )
 		{
 #if defined(_OPENMP)
-#pragma omp atomic
+#include "pragma_omp_atomic_update.h"
 #endif
 		  nmiss++;
 		}
