@@ -92,7 +92,7 @@ void *Ensval(void *argument)
   x = NULL;
   val = NULL;
   weights = NULL; 
-  int vlistCheck, gridsizeCheck;
+  //int vlistCheck, gridsizeCheck;
   
   cdoInitialize(argument);
   
@@ -118,31 +118,31 @@ void *Ensval(void *argument)
   }
 
   // allocate array to hold results 
-  r = malloc ( nostreams*sizeof(double) );
+  r = (double*) malloc( nostreams*sizeof(double));
   
 
   // one stream for each value of the decomposition
-  streamID2 = malloc ( nostreams*sizeof(int) );
-  vlistID2 = malloc ( nostreams*sizeof(int) );
-  taxisID2 = malloc ( nostreams*sizeof(int) );
-  zaxisID2 = malloc ( nostreams*sizeof(int) );
+  streamID2 = (int*) malloc( nostreams*sizeof(int));
+  vlistID2 = (int*) malloc( nostreams*sizeof(int));
+  taxisID2 = (int*) malloc( nostreams*sizeof(int));
+  zaxisID2 = (int*) malloc( nostreams*sizeof(int));
 
-  val = calloc ( nfiles,sizeof(double) );
+  val = (double*) calloc( nfiles,sizeof(double));
   
   if ( operfunc == CRPS ) {
-    alpha=calloc ( nens+1,sizeof(double) );
-    beta =calloc ( nens+1,sizeof(double) );
-    alpha_weights=calloc ( nens+1,sizeof(double) );
-    beta_weights =calloc ( nens+1,sizeof(double) );
+    alpha= (double*) calloc( nens+1,sizeof(double));
+    beta = (double*) calloc( nens+1,sizeof(double));
+    alpha_weights= (double*) calloc( nens+1,sizeof(double));
+    beta_weights = (double*) calloc( nens+1,sizeof(double));
   }
   else if ( operfunc == BRS ) {
-    brs_g = calloc ( nens+1,sizeof(double) );
-    brs_o = calloc ( nens+1,sizeof(double) );
+    brs_g = (double*) calloc( nens+1,sizeof(double));
+    brs_o = (double*) calloc( nens+1,sizeof(double));
   }
   if ( cdoVerbose )
     cdoPrint("Ensemble over %d files (Ensstat5).", nfiles-1);
 
-  ef = malloc(nfiles*sizeof(ens_file_t));
+  ef = (ens_file_t*) malloc(nfiles*sizeof(ens_file_t));
   
   for ( fileID = 0; fileID < nfiles; fileID++ )
     {
@@ -203,7 +203,7 @@ void *Ensval(void *argument)
       break;
     }
 
-    ofilename = calloc(namelen, sizeof(char));
+    ofilename = (char*) calloc(namelen, sizeof(char));
 
     sprintf(ofilename, "%s.%s%s", ofilebase, type_suffix, file_suffix);
     // fprintf(stderr, "StreamID %i: %s\n", stream, ofilename);
@@ -270,11 +270,11 @@ void *Ensval(void *argument)
 		  missval  = vlistInqVarMissval(vlistID1, varID);
 		  gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));//vlistGridsizeMax(vlistID1);
 		  if ( weights ) free(weights); 
-		  weights=malloc (gridsize*sizeof(double));
+		  weights= (double*) malloc(gridsize*sizeof(double));
 		}
 
 	      if (ef[fileID].array ) free(ef[fileID].array);
-	      ef[fileID].array = malloc(gridsize*sizeof(double));
+	      ef[fileID].array = (double*) malloc(gridsize*sizeof(double));
 
 	      streamID = ef[fileID].streamID;
 	      streamReadRecord(streamID, ef[fileID].array, &nmiss);
@@ -437,7 +437,7 @@ void *Ensval(void *argument)
 	      osum += brs_o[k];
 	    }
 	    
-	    if ( abs(osum-1) > 1e-06 || abs(gsum-1) > 1e-06 )  {
+	    if ( fabs(osum-1) > 1.e-06 || fabs(gsum-1) > 1.e-06 )  {
 	      cdoAbort("Internal error - normalization constraint of problem not fulfilled");
 	      cdoAbort("This is likely due to missing values");
 	    }
