@@ -27,9 +27,6 @@
       Gridboxstat    gridboxvar          Gridbox variance
 */
 
-#if defined(_OPENMP)
-#  include <omp.h>
-#endif
 
 #include <cdi.h>
 #include "cdo.h"
@@ -103,19 +100,19 @@ int genBoxGrid(int gridID1, int xinc, int yinc)
       if ( gridInqXbounds(gridID1, NULL) && gridInqYbounds(gridID1, NULL) )
         gridHasBounds = TRUE;
 
-      xvals1 = malloc(nlon1*sizeof(double));
-      yvals1 = malloc(nlat1*sizeof(double));
-      xvals2 = malloc(nlon2*sizeof(double));
-      yvals2 = malloc(nlat2*sizeof(double));
+      xvals1 = (double*) malloc(nlon1*sizeof(double));
+      yvals1 = (double*) malloc(nlat1*sizeof(double));
+      xvals2 = (double*) malloc(nlon2*sizeof(double));
+      yvals2 = (double*) malloc(nlat2*sizeof(double));
       gridInqXvals(gridID1, xvals1);
       gridInqYvals(gridID1, yvals1);
 
       if ( gridHasBounds )
         {
-          grid1_corner_lon = malloc(2*nlon1*sizeof(double));
-          grid1_corner_lat = malloc(2*nlat1*sizeof(double));
-          grid2_corner_lon = malloc(2*nlon2*sizeof(double));
-          grid2_corner_lat = malloc(2*nlat2*sizeof(double));
+          grid1_corner_lon = (double*) malloc(2*nlon1*sizeof(double));
+          grid1_corner_lat = (double*) malloc(2*nlat1*sizeof(double));
+          grid2_corner_lon = (double*) malloc(2*nlon2*sizeof(double));
+          grid2_corner_lat = (double*) malloc(2*nlat2*sizeof(double));
           gridInqXbounds(gridID1, grid1_corner_lon);
           gridInqYbounds(gridID1, grid1_corner_lat);
         }
@@ -177,10 +174,10 @@ int genBoxGrid(int gridID1, int xinc, int yinc)
       if ( gridInqXbounds(gridID1, NULL) && gridInqYbounds(gridID1, NULL) )
         gridHasBounds = TRUE;
       
-      xvals1 = malloc(nlon1*nlat1*sizeof(double));
-      yvals1 = malloc(nlon1*nlat1*sizeof(double));
-      xvals2 = malloc(nlon2*nlat2*sizeof(double));
-      yvals2 = malloc(nlon2*nlat2*sizeof(double));
+      xvals1 = (double*) malloc(nlon1*nlat1*sizeof(double));
+      yvals1 = (double*) malloc(nlon1*nlat1*sizeof(double));
+      xvals2 = (double*) malloc(nlon2*nlat2*sizeof(double));
+      yvals2 = (double*) malloc(nlon2*nlat2*sizeof(double));
       gridInqXvals(gridID1, xvals1);
       gridInqYvals(gridID1, yvals1);
 
@@ -195,10 +192,10 @@ int genBoxGrid(int gridID1, int xinc, int yinc)
       
       if ( gridHasBounds )
         {
-          grid1_corner_lon = malloc(4*nlon1*nlat1*sizeof(double));
-          grid1_corner_lat = malloc(4*nlon1*nlat1*sizeof(double));
-          grid2_corner_lon = malloc(4*nlon2*nlat2*sizeof(double));
-          grid2_corner_lat = malloc(4*nlon2*nlat2*sizeof(double));
+          grid1_corner_lon = (double*) malloc(4*nlon1*nlat1*sizeof(double));
+          grid1_corner_lat = (double*) malloc(4*nlon1*nlat1*sizeof(double));
+          grid2_corner_lon = (double*) malloc(4*nlon2*nlat2*sizeof(double));
+          grid2_corner_lat = (double*) malloc(4*nlon2*nlat2*sizeof(double));
           gridInqXbounds(gridID1, grid1_corner_lon);
           gridInqYbounds(gridID1, grid1_corner_lat);
 
@@ -533,7 +530,6 @@ void gridboxstat(field_t *field1, field_t *field2, int xinc, int yinc, int statf
   double missval;
   long ig, i, j, ii, jj, index;
   long gridsize;
-  long ompthID;
   field_t *field;
   int isize;
   int useWeight = FALSE;
@@ -545,14 +541,14 @@ void gridboxstat(field_t *field1, field_t *field2, int xinc, int yinc, int statf
   if ( field1->weight ) useWeight = TRUE;
 
   gridsize      = xinc*yinc;
-  field = malloc(ompNumThreads*sizeof(field_t));
+  field = (field_t*) malloc(ompNumThreads*sizeof(field_t));
   for ( i = 0; i < ompNumThreads; i++ )
     {
       field[i].size    = gridsize;
-      field[i].ptr     = malloc(gridsize*sizeof(double));
+      field[i].ptr     = (double*) malloc(gridsize*sizeof(double));
       field[i].weight  = NULL;
       if ( useWeight )
-	field[i].weight  = malloc(gridsize*sizeof(double));
+	field[i].weight  = (double*) malloc(gridsize*sizeof(double));
       field[i].missval = field1->missval;
       field[i].nmiss   = 0;
     }
@@ -570,22 +566,19 @@ void gridboxstat(field_t *field1, field_t *field2, int xinc, int yinc, int statf
   nlat2 = gridInqYsize(gridID2);
 
 #if defined(_OPENMP)
-#pragma omp parallel for default(shared) private(ig, ilat, ilon, j, jj, i, ii, index, isize, ompthID)
+#pragma omp parallel for default(shared) private(ig, ilat, ilon, j, jj, i, ii, index, isize)
 #endif
   for ( ig = 0; ig < nlat2*nlon2; ++ig )
     {
-#if defined(_OPENMP)
-      ompthID = omp_get_thread_num();
-#else
-      ompthID = 0;
-#endif
+      int ompthID = cdo_omp_get_thread_num();
+
       /*
       int lprogress = 1;
 #if defined(_OPENMP)
       if ( ompthID != 0 ) lprogress = 0;
 #endif
 #if defined(_OPENMP)
-#pragma omp atomic
+#include "pragma_omp_atomic_update.h"
 #endif
       findex++;
       if ( lprogress ) progressStatus(0, 1, findex/nlat2*nlon2);
@@ -639,16 +632,16 @@ void *Gridboxstat(void *argument)
   int vlistID1, vlistID2;
   int lastgrid = -1;
   int wstatus = FALSE;
-  int code = 0, oldcode = 0;
   int index, ngrids;
   int recID, nrecs;
   int tsID, varID, levelID;
   int needWeights = FALSE;
   int gridID1, gridID2;
   int gridsize1, gridsize2;
-  field_t field1, field2;
   int taxisID1, taxisID2;
   int xinc, yinc;
+  field_t field1, field2;
+  char varname[CDI_MAX_NAME];
 
   cdoInitialize(argument);
 
@@ -700,13 +693,13 @@ void *Gridboxstat(void *argument)
   field_init(&field2);
 
   gridsize1 = gridInqSize(gridID1);
-  field1.ptr    = malloc(gridsize1*sizeof(double));
+  field1.ptr    = (double*) malloc(gridsize1*sizeof(double));
   field1.weight = NULL;
   if ( needWeights )
-    field1.weight = malloc(gridsize1*sizeof(double));
+    field1.weight = (double*) malloc(gridsize1*sizeof(double));
 
   gridsize2 = gridInqSize(gridID2);
-  field2.ptr    = malloc(gridsize2*sizeof(double));
+  field2.ptr    = (double*) malloc(gridsize2*sizeof(double));
   field2.weight = NULL;
 
   tsID = 0;
@@ -734,9 +727,11 @@ void *Gridboxstat(void *argument)
 	      lastgrid = field1.grid;
               wstatus = gridWeights(field1.grid, field1.weight);
             }
-          code = vlistInqVarCode(vlistID1, varID);
-          if ( wstatus != 0 && tsID == 0 && code != oldcode )
-            cdoWarning("Using constant grid cell area weights for code %d!", oldcode=code);
+          if ( wstatus != 0 && tsID == 0 && levelID == 0 )
+	    {
+	      vlistInqVarName(vlistID1, varID, varname);
+	      cdoWarning("Using constant grid cell area weights for variable %s!", varname);
+	    }
           
           gridboxstat(&field1, &field2, xinc, yinc, operfunc);
           
