@@ -228,18 +228,16 @@ void *Intlevel3d(void *argument)
 
   operatorID = cdoOperatorID();
 
-  if ( cdoVerbose ) 
-    cdoPrint("Called '%s'\n",__func__);
-
-  if ( operatorID == INTLEVELX3D ) expol = TRUE;
+  if      ( operatorID == INTLEVEL3D )  expol = FALSE;
+  else if ( operatorID == INTLEVELX3D ) expol = TRUE;
 
   operatorInputArg("icoordinate");
 
   /*  Read filename from Parameter */
-  operatorInputArg("grid description file or name, remap file (SCRIP netCDF)");
+  operatorInputArg("filename for vertical source coordinates variable");
   operatorCheckArgc(1);
   argument_t *fileargument = file_argument_new(operatorArgv()[0]);
-  streamID0 = streamOpenRead(fileargument);                /*  3d vertical input coordinate */
+  streamID0 = streamOpenRead(fileargument);                     /*  3d vertical input coordinate */
   file_argument_free(fileargument);
   streamID1 = streamOpenRead(cdoStreamName(0));                 /*  input data */
   streamID2 = streamOpenRead(cdoStreamName(1));                 /*  3d target vertical coordinate */
@@ -257,8 +255,8 @@ void *Intlevel3d(void *argument)
    */
   {
     nvars = vlistNvars(vlistID0);
-    if (nvars != 1) 
-      cdoAbort("Only one single variable is allowed!");
+    if (nvars != 1) cdoAbort("Only one single variable is allowed!");
+
     gridID     = vlistInqVarGrid(vlistID0, 0);
     zaxisID    = vlistInqVarZaxis(vlistID0, 0);
     gridsize   = gridInqSize(gridID);
@@ -266,19 +264,18 @@ void *Intlevel3d(void *argument)
 
     nlonIn  = gridInqXsize(gridID);
     nlatIn  = gridInqYsize(gridID);
-    lonIn   = malloc(nlonIn*sizeof(double));
-    latIn   = malloc(nlatIn*sizeof(double));
+    lonIn   = (double*) malloc(nlonIn*sizeof(double));
+    latIn   = (double*) malloc(nlatIn*sizeof(double));
     gridInqXvals(gridID, lonIn);
     gridInqYvals(gridID, latIn);
 
-    zlevels_in = malloc(gridsize*(nlevel+2)*sizeof(double*));
+    zlevels_in = (double*) malloc(gridsize*(nlevel+2)*sizeof(double));
     nlevi      = nlevel;   /* number of input levels for later use */
     gridsizei  = gridsize; /* horizontal gridsize of input z coordinate */
     nrecs      = streamInqTimestep(streamID0, 0);
-    if (cdoVerbose)
-      cdoPrint("%d records input 3d vertical height\n",nrecs);
-    {
-      for ( recID = 0; recID < nrecs; recID++ )
+    if (cdoVerbose) cdoPrint("%d records input 3d vertical height\n",nrecs);
+
+    for ( recID = 0; recID < nrecs; recID++ )
       {
         streamInqRecord(streamID0, &varID, &levelID);
         gridsize = gridInqSize(vlistInqVarGrid(vlistID0, varID));
@@ -286,7 +283,6 @@ void *Intlevel3d(void *argument)
         single1  = zlevels_in + offset;
         streamReadRecord(streamID0, single1, &zlevels_in_miss);
       }
-    }
   }
 
   /*
@@ -302,26 +298,25 @@ void *Intlevel3d(void *argument)
 
     nlonOut = gridInqXsize(gridID);
     nlatOut = gridInqYsize(gridID);
-    lonOut  = malloc(nlonOut*sizeof(double));
-    latOut  = malloc(nlatOut*sizeof(double));
+    lonOut  = (double*) malloc(nlonOut*sizeof(double));
+    latOut  = (double*) malloc(nlatOut*sizeof(double));
     gridInqXvals(gridID, lonOut);
     gridInqYvals(gridID, latOut);
 
-    zlevels_out = malloc(gridsize*nlevel*sizeof(double));
+    zlevels_out = (double*) malloc(gridsize*nlevel*sizeof(double));
     nlevo       = nlevel;  /* number of output levels for later use */
     gridsizeo   = gridsize;/* horizontal gridsize of output z coordinate */
     nrecs       = streamInqTimestep(streamID2, 0);
-    if (cdoVerbose)
-      cdoPrint("%d records target 3d vertical height and gridsize %d\n",nrecs,gridsize);
+    if (cdoVerbose) cdoPrint("%d records target 3d vertical height and gridsize %d\n",nrecs,gridsize);
 
     for ( recID = 0; recID < nrecs; recID++ )
-    {
-      streamInqRecord(streamID2, &varID, &levelID);
-      gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
-      offset   = gridsize*levelID;
-      single1  = zlevels_out + offset;
-      streamReadRecord(streamID2, single1, &zlevels_out_miss);
-    }
+      {
+	streamInqRecord(streamID2, &varID, &levelID);
+	gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
+	offset   = gridsize*levelID;
+	single1  = zlevels_out + offset;
+	streamReadRecord(streamID2, single1, &zlevels_out_miss);
+      }
   }
 
   /* Missing values are not allowed for coordinate variables */
@@ -331,6 +326,7 @@ void *Intlevel3d(void *argument)
     {
       if ( cdoVerbose ) cdoPrint("Input vertical coordinate has no missing values.");
     }
+
   if ( 0 != zlevels_out_miss )
     cdoAbort("Output vertical coordinate variables are not allowd to contian missing values.");
   else
@@ -345,10 +341,10 @@ void *Intlevel3d(void *argument)
    */
   if ( gridsizei != gridsizeo )
     cdoAbort("Input and output vertical coordinate must have the same gridsize");
-   gridSize = gridsizeo;
 
-   /* input and output vertical coordinates must have exactly the same
-    * horizontal grid */
+  gridSize = gridsizeo;
+
+   /* input and output vertical coordinates must have exactly the same horizontal grid */
    if ( nlonIn != nlonOut || 
         nlatIn != nlatOut ||
         memcmp(lonIn,lonOut,nlonIn*sizeof(double)) ||
@@ -427,10 +423,10 @@ void *Intlevel3d(void *argument)
   /*
    * Create weights for later interpolation - assumption: input vertical correct is constant in time
    */
-  lev_idx1 = malloc(nlevo*gridSize*sizeof(int));
-  lev_idx2 = malloc(nlevo*gridSize*sizeof(int));
-  lev_wgt1 = malloc(nlevo*gridSize*sizeof(double));
-  lev_wgt2 = malloc(nlevo*gridSize*sizeof(double));
+  lev_idx1 = (int*) malloc(nlevo*gridSize*sizeof(int));
+  lev_idx2 = (int*) malloc(nlevo*gridSize*sizeof(int));
+  lev_wgt1 = (double*) malloc(nlevo*gridSize*sizeof(double));
+  lev_wgt2 = (double*) malloc(nlevo*gridSize*sizeof(double));
 
   gen_weights3d(expol, nlevi+2, gridSize, zlevels_in, nlevo, zlevels_out, lev_idx1, lev_idx2, lev_wgt1, lev_wgt2);
 
@@ -438,7 +434,7 @@ void *Intlevel3d(void *argument)
    * Copy z-axis information to output z-axis
    */
   zaxisID3 = zaxisCreate(zaxisInqType(zaxisID1), nlevo);
-  lev2 = malloc(nlevo*sizeof(double));
+  lev2 = (double*) malloc(nlevo*sizeof(double));
   /* fill values with its indices */
   for (i=0;i<nlevo;i++)
     lev2[i] = (double) i;
@@ -470,11 +466,11 @@ void *Intlevel3d(void *argument)
 
   maxlev   = nlevi > nlevo ? nlevi : nlevo;
   nvars     = vlistNvars(vlistID1);
-  vars      = malloc(nvars*sizeof(int));
-  vardata1  = malloc(nvars*sizeof(double*)); /* input                                         */
-  vardata2  = malloc(nvars*sizeof(double*)); /* output                                        */
-  varnmiss  = malloc(nvars*sizeof(int*));    /* can for missing values of arbitrary variables */
-  varinterp = malloc(nvars*sizeof(int));     /* marker for variables to be interpolated       */
+  vars      = (int*) malloc(nvars*sizeof(int));
+  vardata1  = (double**) malloc(nvars*sizeof(double*)); /* input                                         */
+  vardata2  = (double**) malloc(nvars*sizeof(double*)); /* output                                        */
+  varnmiss  = (int**) malloc(nvars*sizeof(int*));    /* can for missing values of arbitrary variables */
+  varinterp = (int*) malloc(nvars*sizeof(int));     /* marker for variables to be interpolated       */
 
   /* by default no variable should be interpolated */
   for (i = 0; i < nvars;i++)
@@ -489,7 +485,7 @@ void *Intlevel3d(void *argument)
 
       vlistInqVarName(vlistID1, varID, varname);
 
-      vardata1[varID] = malloc(gridsize*nlevel*sizeof(double));
+      vardata1[varID] = (double*) malloc(gridsize*nlevel*sizeof(double));
 
       /*  variabls for interpolation:
        *  * have the required vertical axis, i.e. the correct number of levels (nlevi)
@@ -500,8 +496,8 @@ void *Intlevel3d(void *argument)
         {
           nlonIn  = gridInqXsize(gridID);
           nlatIn  = gridInqYsize(gridID);
-          lonIn   = malloc(nlonIn*sizeof(double));
-          latIn   = malloc(nlatIn*sizeof(double));
+          lonIn   = (double*) malloc(nlonIn*sizeof(double));
+          latIn   = (double*) malloc(nlatIn*sizeof(double));
           gridInqXvals(gridID, lonIn);
           gridInqYvals(gridID, latIn);
 
@@ -512,14 +508,14 @@ void *Intlevel3d(void *argument)
             {
               varinterp[varID] = FALSE;
               vardata2[varID]  = vardata1[varID];
-              varnmiss[varID]  = malloc(nlevel*sizeof(int));
+              varnmiss[varID]  = (int*) malloc(nlevel*sizeof(int));
               if ( cdoVerbose ) cdoPrint("Ignore variable %s with %d levels\n",varname,nlevel);
             }
           else
             {
               varinterp[varID] = TRUE;
-              vardata2[varID]  = malloc(gridsize*nlevo*sizeof(double));
-              varnmiss[varID]  = malloc(maxlev*sizeof(int));
+              vardata2[varID]  = (double*) malloc(gridsize*nlevo*sizeof(double));
+              varnmiss[varID]  = (int*) malloc(maxlev*sizeof(int));
               memset(varnmiss[varID], 0, maxlev*sizeof(int));
             }
         }
@@ -527,7 +523,7 @@ void *Intlevel3d(void *argument)
         {
           varinterp[varID] = FALSE;
           vardata2[varID]  = vardata1[varID];
-          varnmiss[varID]  = malloc(nlevel*sizeof(int));
+          varnmiss[varID]  = (int*) malloc(nlevel*sizeof(int));
           if ( cdoVerbose ) cdoPrint("Ignore variable %s with %d levels\n",varname,nlevel);
         }
       }
