@@ -29,24 +29,24 @@
 
 #include "printinfo.h"
 
-#define MAXCHARS 82
-
 const char * tunit2str(int tunits)
 {
-  if      ( tunits == TUNIT_YEAR )    return ("years");
-  else if ( tunits == TUNIT_MONTH )   return ("months");
-  else if ( tunits == TUNIT_DAY )     return ("days");
-  else if ( tunits == TUNIT_12HOURS ) return ("12hours");
-  else if ( tunits == TUNIT_6HOURS )  return ("6hours");
-  else if ( tunits == TUNIT_3HOURS )  return ("3hours");
-  else if ( tunits == TUNIT_HOUR )    return ("hours");
-  else if ( tunits == TUNIT_MINUTE )  return ("minutes");
-  else if ( tunits == TUNIT_SECOND )  return ("seconds");
-  else                                return ("unknown");
+  if      ( tunits == TUNIT_YEAR )       return ("years");
+  else if ( tunits == TUNIT_MONTH )      return ("months");
+  else if ( tunits == TUNIT_DAY )        return ("days");
+  else if ( tunits == TUNIT_12HOURS )    return ("12hours");
+  else if ( tunits == TUNIT_6HOURS )     return ("6hours");
+  else if ( tunits == TUNIT_3HOURS )     return ("3hours");
+  else if ( tunits == TUNIT_HOUR )       return ("hours");
+  else if ( tunits == TUNIT_30MINUTES )  return ("30minutes");
+  else if ( tunits == TUNIT_QUARTER )    return ("15minutes");
+  else if ( tunits == TUNIT_MINUTE )     return ("minutes");
+  else if ( tunits == TUNIT_SECOND )     return ("seconds");
+  else                                   return ("unknown");
 }
 
 
-const char * calendar2str(int calendar)
+const char* calendar2str(int calendar)
 {
   if      ( calendar == CALENDAR_STANDARD )  return ("standard");
   else if ( calendar == CALENDAR_PROLEPTIC ) return ("proleptic_gregorian");
@@ -56,6 +56,22 @@ const char * calendar2str(int calendar)
   else                                       return ("unknown");
 }
 
+static
+void limit_string_length(char* string)
+{
+  size_t len = strlen(string);
+
+  if ( len > 10 )
+    {
+      for ( size_t i = 3; i < len; ++i )
+	if ( string[i] == ' ' )
+	  {
+	    string[i] = 0;
+	    break;
+	  }
+    }
+}
+
 
 void *Sinfo(void *argument)
 {
@@ -66,21 +82,15 @@ void *Sinfo(void *argument)
   int varID;
   int gridsize = 0;
   int gridID, zaxisID, code, tabnum, param;
-  int zaxistype, ltype;
   int vdate, vtime;
-  int nrecs, nvars, nzaxis, ntsteps;
-  int levelID, levelsize;
-  int tsID, ntimeout;
+  int nvars, ntsteps;
+  int levelsize;
   int tsteptype, taxisID;
-  int nbyte, nbyte0;
-  int index;
+  char tmpname[CDI_MAX_NAME];
   char varname[CDI_MAX_NAME];
-  char longname[CDI_MAX_NAME];
-  char units[CDI_MAX_NAME];
   char paramstr[32];
   char vdatestr[32], vtimestr[32];
-  double level;
-  char *modelptr, *instptr;
+  const char *modelptr, *instptr;
   int streamID = 0;
   int vlistID;
   int datatype;
@@ -108,19 +118,24 @@ void *Sinfo(void *argument)
 
       vlistID = streamInqVlist(streamID);
 
-      fprintf(stdout, "   File format: ");
+      set_text_color(stdout, BRIGHT, BLACK);
+      fprintf(stdout, "   File format");
+      reset_text_color(stdout);
+      fprintf(stdout, " : ");
       printFiletype(streamID, vlistID);
 
+      set_text_color(stdout, BRIGHT, BLACK);
       if ( lensemble )
-	fprintf(stdout, "%6d : Institut Source   Ttype    Einfo Levels Num  Gridsize Num Dtype : ",  -(indf+1));
+	fprintf(stdout, "%6d : Institut Source   Ttype    Einfo Levels Num    Points Num Dtype : ",  -(indf+1));
       else
-	fprintf(stdout, "%6d : Institut Source   Ttype    Levels Num  Gridsize Num Dtype : ",  -(indf+1));
+	fprintf(stdout, "%6d : Institut Source   Ttype    Levels Num    Points Num Dtype : ",  -(indf+1));
 
       if      ( operfunc == func_name ) fprintf(stdout, "Parameter name");
       else if ( operfunc == func_code ) fprintf(stdout, "Table Code");
       else                              fprintf(stdout, "Parameter ID");
 
       if ( cdoVerbose ) fprintf(stdout, " : Extra" );              
+      reset_text_color(stdout);
       fprintf(stdout, "\n" );              
 
       nvars = vlistNvars(vlistID);
@@ -133,31 +148,27 @@ void *Sinfo(void *argument)
 	  gridID  = vlistInqVarGrid(vlistID, varID);
 	  zaxisID = vlistInqVarZaxis(vlistID, varID);
 
-	  fprintf(stdout, "%6d : ", varID + 1);
-
+	  set_text_color(stdout, BRIGHT, BLACK);
+	  fprintf(stdout, "%6d", varID+1);
+	  reset_text_color(stdout);
+	  set_text_color(stdout, RESET, BLACK);
+	  fprintf(stdout, " : ");
+	  reset_text_color(stdout);
+	      
+	  set_text_color(stdout, RESET, BLUE);
 	  /* institute info */
 	  instptr = institutInqNamePtr(vlistInqVarInstitut(vlistID, varID));
-	  if ( instptr )
-	    fprintf(stdout, "%-8s ", instptr);
-	  else
-	    fprintf(stdout, "unknown  ");
+	  strcpy(tmpname, "unknown");
+	  if ( instptr ) strcpy(tmpname, instptr);
+	  limit_string_length(tmpname);
+	  fprintf(stdout, "%-8s ", tmpname);
 
 	  /* source info */
 	  modelptr = modelInqNamePtr(vlistInqVarModel(vlistID, varID));
-	  if ( modelptr )
-	    {
-	      size_t len = strlen(modelptr);
-	      if ( len > 10 )
-		for ( size_t i = 3; i < len; ++i )
-		  if ( modelptr[i] == ' ' )
-		    {
-		      modelptr[i] = 0;
-		      break;
-		    }
-	      fprintf(stdout, "%-8s ", modelptr);
-	    }
-	  else
-	    fprintf(stdout, "unknown  ");
+	  strcpy(tmpname, "unknown");
+	  if ( modelptr ) strcpy(tmpname, modelptr);
+	  limit_string_length(tmpname);
+	  fprintf(stdout, "%-8s ", tmpname);
 
 	  /* tsteptype */
 	  tsteptype = vlistInqVarTsteptype(vlistID, varID);
@@ -167,7 +178,10 @@ void *Sinfo(void *argument)
 	  else if ( tsteptype == TSTEP_INSTANT3 ) fprintf(stdout, "%-8s ", "instant");
 	  else if ( tsteptype == TSTEP_MIN      ) fprintf(stdout, "%-8s ", "min");
 	  else if ( tsteptype == TSTEP_MAX      ) fprintf(stdout, "%-8s ", "max");
+	  else if ( tsteptype == TSTEP_AVG      ) fprintf(stdout, "%-8s ", "avg");
 	  else if ( tsteptype == TSTEP_ACCUM    ) fprintf(stdout, "%-8s ", "accum");
+	  else if ( tsteptype == TSTEP_RANGE    ) fprintf(stdout, "%-8s ", "range");
+	  else if ( tsteptype == TSTEP_DIFF     ) fprintf(stdout, "%-8s ", "diff");
 	  else                                    fprintf(stdout, "%-8s ", "unknown");
 
 	  /* ensemble information */
@@ -201,19 +215,25 @@ void *Sinfo(void *argument)
 	  else
 	    fprintf(stdout, "z ");
 
-	  /* parameter info */
+	  reset_text_color(stdout);
+	      
+	  set_text_color(stdout, RESET, BLACK);
 	  fprintf(stdout, ": ");
+	  reset_text_color(stdout);
 
+	  /* parameter info */
 	  cdiParamToString(param, paramstr, sizeof(paramstr));
 
 	  if ( operfunc == func_name ) vlistInqVarName(vlistID, varID, varname);
 
+	  set_text_color(stdout, BRIGHT, GREEN);
 	  if ( operfunc == func_name )
 	    fprintf(stdout, "%-14s", varname);
 	  else if ( operfunc == func_code )
 	    fprintf(stdout, "%4d %4d   ", tabnum, code);
 	  else
 	    fprintf(stdout, "%-14s", paramstr);
+	  reset_text_color(stdout);
 
 	  if ( cdoVerbose )
 	    {
@@ -225,100 +245,37 @@ void *Sinfo(void *argument)
 	  fprintf(stdout, "\n");
 	}
 
-      fprintf(stdout, "   Grid coordinates :\n");
+      set_text_color(stdout, BRIGHT, BLACK);
+      fprintf(stdout, "   Grid coordinates");
+      reset_text_color(stdout);
+      fprintf(stdout, " :\n");
+
       printGridInfo(vlistID);
 
-      nzaxis = vlistNzaxis(vlistID);
-      fprintf(stdout, "   Vertical coordinates :\n");
-      for ( index = 0; index < nzaxis; index++)
-	{
-	  zaxisID   = vlistZaxis(vlistID, index);
-	  zaxistype = zaxisInqType(zaxisID);
-	  ltype     = zaxisInqLtype(zaxisID);
-	  levelsize = zaxisInqSize(zaxisID);
-	  /* zaxisInqLongname(zaxisID, longname); */
-	  zaxisName(zaxistype, longname);
-	  longname[18] = 0;
-	  zaxisInqUnits(zaxisID, units);
-	  units[12] = 0;
-	  if ( zaxistype == ZAXIS_GENERIC && ltype != 0 )
-	    nbyte0    = fprintf(stdout, "  %4d : %-11s  (ltype=%3d) : ", vlistZaxisIndex(vlistID, zaxisID)+1, longname, ltype);
-	  else
-	    nbyte0    = fprintf(stdout, "  %4d : %-18s %5s : ", vlistZaxisIndex(vlistID, zaxisID)+1, longname, units);
-	  nbyte = nbyte0;
-	  for ( levelID = 0; levelID < levelsize; levelID++ )
-	    {
-	      if ( nbyte > MAXCHARS )
-		{
-		  fprintf(stdout, "\n");
-		  fprintf(stdout, "%*s", nbyte0, "");
-		  nbyte = nbyte0;
-		}
-	      level = zaxisInqLevel(zaxisID, levelID);
-	      nbyte += fprintf(stdout, "%.9g ", level);
-	    }
-	  fprintf(stdout, "\n");
-	  if ( zaxisInqLbounds(zaxisID, NULL) && zaxisInqUbounds(zaxisID, NULL) )
-	    {
-	      double level1, level2;
-	      nbyte = nbyte0;
-	      fprintf(stdout, "%33s : ", "bounds");
-	      for ( levelID = 0; levelID < levelsize; levelID++ )
-		{
-		  if ( nbyte > MAXCHARS )
-		    {
-		      fprintf(stdout, "\n");
-		      fprintf(stdout, "%*s", nbyte0, "");
-		      nbyte = nbyte0;
-		    }
-		  level1 = zaxisInqLbound(zaxisID, levelID);
-		  level2 = zaxisInqUbound(zaxisID, levelID);
-		  nbyte += fprintf(stdout, "%.9g-%.9g ", level1, level2);
-		}
-	      fprintf(stdout, "\n");
-	    }
+      set_text_color(stdout, BRIGHT, BLACK);
+      fprintf(stdout, "   Vertical coordinates");
+      reset_text_color(stdout);
+      fprintf(stdout, " :\n");
 
-          if ( zaxistype == ZAXIS_REFERENCE )
-            {
-              int number   = zaxisInqNumber(zaxisID);
-
-              if ( number > 0 )
-                {
-                  fprintf(stdout, "%33s : ", "zaxis");
-                  fprintf(stdout, "number = %d\n", number);
-                }
-
-              char uuidOfVGrid[17];
-              zaxisInqUUID(zaxisID, uuidOfVGrid);
-              if ( uuidOfVGrid[0] != 0 )
-                {
-                  char uuidOfVGridStr[37];
-                  uuid2str(uuidOfVGrid, uuidOfVGridStr);
-                  if ( uuidOfVGridStr[0] != 0  && strlen(uuidOfVGridStr) == 36 )
-                    {
-                      fprintf(stdout, "%33s : ", "uuid");
-                      fprintf(stdout, "%s\n", uuidOfVGridStr);
-                    }
-                }
-            }
-	}
+      printZaxisInfo(vlistID);
 
       taxisID = vlistInqTaxis(vlistID);
       ntsteps = vlistNtsteps(vlistID);
 
       if ( ntsteps != 0 )
 	{
+	  set_text_color(stdout, BRIGHT, BLACK);
+	  fprintf(stdout, "   Time coordinate");
+	  reset_text_color(stdout);
 	  if ( ntsteps == CDI_UNDEFID )
-	    fprintf(stdout, "   Time coordinate :  unlimited steps\n");
+	    fprintf(stdout, " :  unlimited steps\n");
 	  else
-	    fprintf(stdout, "   Time coordinate :  %d step%s\n", ntsteps, ntsteps == 1 ? "" : "s");
+	    fprintf(stdout, " :  %d step%s\n", ntsteps, ntsteps == 1 ? "" : "s");
 
 	  if ( taxisID != CDI_UNDEFID )
 	    {
-	      if ( taxisInqType(taxisID) == TAXIS_RELATIVE )
+	      if ( taxisInqType(taxisID) != TAXIS_ABSOLUTE )
 		{
-		  int calendar, tunits;
-
 		  vdate = taxisInqRdate(taxisID);
 		  vtime = taxisInqRtime(taxisID);
 
@@ -327,42 +284,38 @@ void *Sinfo(void *argument)
 
 		  fprintf(stdout, "     RefTime = %s %s", vdatestr, vtimestr);
 		      
-		  tunits = taxisInqTunit(taxisID);
+		  int tunits = taxisInqTunit(taxisID);
 		  if ( tunits != CDI_UNDEFID )  fprintf(stdout, "  Units = %s", tunit2str(tunits));
 	      
-		  calendar = taxisInqCalendar(taxisID);
+		  int calendar = taxisInqCalendar(taxisID);
 		  if ( calendar != CDI_UNDEFID )  fprintf(stdout, "  Calendar = %s", calendar2str(calendar));
 
 		  if ( taxisHasBounds(taxisID) )
 		    fprintf(stdout, "  Bounds = true");
 
 		  fprintf(stdout, "\n");
+
+		  if ( taxisInqType(taxisID) == TAXIS_FORECAST )
+		    {
+		      vdate = taxisInqFdate(taxisID);
+		      vtime = taxisInqFtime(taxisID);
+
+		      date2str(vdate, vdatestr, sizeof(vdatestr));
+		      time2str(vtime, vtimestr, sizeof(vtimestr));
+
+		      fprintf(stdout, "     ForecastRefTime = %s %s", vdatestr, vtimestr);
+		      fprintf(stdout, "\n");
+		    }
 		}
 	    }
 
 	  fprintf(stdout, "  YYYY-MM-DD hh:mm:ss  YYYY-MM-DD hh:mm:ss  YYYY-MM-DD hh:mm:ss  YYYY-MM-DD hh:mm:ss\n");
 
-	  ntimeout = 0;
-	  tsID = 0;
-	  while ( (nrecs = streamInqTimestep(streamID, tsID)) )
-	    {
-	      if ( ntimeout == 4 )
-		{
-		  ntimeout = 0;
-		  fprintf(stdout, "\n");
-		}
+	  set_text_color(stdout, RESET, MAGENTA);
 
-	      vdate = taxisInqVdate(taxisID);
-	      vtime = taxisInqVtime(taxisID);
+	  printTimesteps(streamID, taxisID, cdoVerbose);
 
-	      date2str(vdate, vdatestr, sizeof(vdatestr));
-	      time2str(vtime, vtimestr, sizeof(vtimestr));
-
-	      fprintf(stdout, " %s %s", vdatestr, vtimestr);
-
-	      ntimeout++;
-	      tsID++;
-	    }
+	  reset_text_color(stdout);
 	  fprintf(stdout, "\n");
 	}
 
