@@ -86,11 +86,16 @@ void *Splittime(void *argument)
 
   cdoInitialize(argument);
 
+  if ( processSelf() != 0 ) cdoAbort("This operator can't be combined with other operators!");
+
   SPLITHOUR = cdoOperatorAdd("splithour", func_time, 10000, NULL);
   SPLITDAY  = cdoOperatorAdd("splitday",  func_date,     1, NULL);
   SPLITMON  = cdoOperatorAdd("splitmon",  func_date,   100, NULL);
   SPLITSEAS = cdoOperatorAdd("splitseas", func_date,   100, NULL);
 
+  UNUSED(SPLITDAY);
+  UNUSED(SPLITHOUR);
+
   operatorID = cdoOperatorID();
   operfunc   = cdoOperatorF1(operatorID);
   operintval = cdoOperatorF2(operatorID);
@@ -128,7 +133,7 @@ void *Splittime(void *argument)
     {
       gridsize = vlistGridsizeMax(vlistID1);
       if ( vlistNumber(vlistID1) != CDI_REAL ) gridsize *= 2;
-      array = malloc(gridsize*sizeof(double));
+      array = (double*) malloc(gridsize*sizeof(double));
     }
 
   nvars = vlistNvars(vlistID1);
@@ -138,7 +143,7 @@ void *Splittime(void *argument)
 
   if ( nconst )
     {
-      vars = malloc(nvars*sizeof(field_t *));
+      vars = (field_t **) malloc(nvars*sizeof(field_t *));
 
       for ( varID = 0; varID < nvars; varID++ )
 	{
@@ -148,13 +153,13 @@ void *Splittime(void *argument)
 	      nlevel  = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
 	      gridsize = gridInqSize(gridID);
 		  
-	      vars[varID] = malloc(nlevel*sizeof(field_t));
+	      vars[varID] = (field_t*) malloc(nlevel*sizeof(field_t));
 
 	      for ( levelID = 0; levelID < nlevel; levelID++ )
 		{
 		  field_init(&vars[varID][levelID]);
 		  vars[varID][levelID].grid    = gridID;
-		  vars[varID][levelID].ptr     = malloc(gridsize*sizeof(double));
+		  vars[varID][levelID].ptr     = (double*) malloc(gridsize*sizeof(double));
 		}
 	    }
 	}
