@@ -187,8 +187,9 @@ void *Timstat(void *argument)
   if ( cdoOperatorF2(operatorID) == 31 ) vlistDefNtsteps(vlistID2, 1);
 
   taxisID1 = vlistInqTaxis(vlistID1);
-  taxis_has_bounds = taxisHasBounds(taxisID1);
   taxisID2 = taxisDuplicate(taxisID1);
+  if ( taxisInqType(taxisID2) == TAXIS_FORECAST ) taxisDefType(taxisID2, TAXIS_RELATIVE);
+  taxis_has_bounds = taxisHasBounds(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
   streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
@@ -225,14 +226,14 @@ void *Timstat(void *argument)
       streamDefVlist(streamID3, vlistID3);
     }
 
-  recVarID   = malloc(nrecords*sizeof(int));
-  recLevelID = malloc(nrecords*sizeof(int));
+  recVarID   = (int*) malloc(nrecords*sizeof(int));
+  recLevelID = (int*) malloc(nrecords*sizeof(int));
 
   gridsize = vlistGridsizeMax(vlistID1);
   if ( vlistNumber(vlistID1) != CDI_REAL ) gridsize *= 2;
 
   field_init(&field);
-  field.ptr = malloc(gridsize*sizeof(double));
+  field.ptr = (double*) malloc(gridsize*sizeof(double));
 
   vars1 = field_malloc(vlistID1, FIELD_PTR);
   samp1 = field_malloc(vlistID1, FIELD_NONE);
@@ -254,10 +255,12 @@ void *Timstat(void *argument)
 	      taxisInqVdateBounds(taxisID1, &date_lb, &date_ub);
 	      taxisInqVtimeBounds(taxisID1, &time_lb, &time_ub);
 	      if ( nsets == 0 )
-		{
-		  vdate_lb = date_lb;
-		  vtime_lb = time_lb;
-		}
+		{ vdate_lb = date_lb; vtime_lb = time_lb; }
+	    }
+	  else
+	    {
+	      if ( nsets == 0 )
+		{ vdate_lb = vdate; vtime_lb = vtime; }
 	    }
 
 	  if ( nsets == 0 ) SET_DATE(indate2, vdate, vtime);
@@ -266,10 +269,9 @@ void *Timstat(void *argument)
 	  if ( DATE_IS_NEQ(indate1, indate2, cmplen) ) break;
 
 	  if ( taxis_has_bounds )
-	    {
-	      vdate_ub = date_ub;
-	      vtime_ub = time_ub;
-	    }
+	    { vdate_ub = date_ub; vtime_ub = time_ub; }
+	  else
+	    { vdate_ub = vdate; vtime_ub = vtime; }
 
 	  for ( recID = 0; recID < nrecs; recID++ )
 	    {
@@ -292,7 +294,7 @@ void *Timstat(void *argument)
 		  if ( nmiss > 0 || samp1[varID][levelID].ptr )
 		    {
 		      if ( samp1[varID][levelID].ptr == NULL )
-			samp1[varID][levelID].ptr = malloc(nwpv*gridsize*sizeof(double));
+			samp1[varID][levelID].ptr = (double*) malloc(nwpv*gridsize*sizeof(double));
 
 		      for ( i = 0; i < nwpv*gridsize; i++ )
 			if ( DBL_IS_EQUAL(vars1[varID][levelID].ptr[i], vars1[varID][levelID].missval) )
@@ -311,7 +313,7 @@ void *Timstat(void *argument)
 		    {
 		      if ( samp1[varID][levelID].ptr == NULL )
 			{
-			  samp1[varID][levelID].ptr = malloc(nwpv*gridsize*sizeof(double));
+			  samp1[varID][levelID].ptr = (double*) malloc(nwpv*gridsize*sizeof(double));
 			  for ( i = 0; i < nwpv*gridsize; i++ )
 			    samp1[varID][levelID].ptr[i] = nsets;
 			}
@@ -429,22 +431,16 @@ void *Timstat(void *argument)
 
       taxisDefVdate(taxisID2, vdate0);
       taxisDefVtime(taxisID2, vtime0);
-      if ( taxis_has_bounds )
-	{
-	  taxisDefVdateBounds(taxisID2, vdate_lb, vdate_ub);
-	  taxisDefVtimeBounds(taxisID2, vtime_lb, vtime_ub);
-	}
+      taxisDefVdateBounds(taxisID2, vdate_lb, vdate_ub);
+      taxisDefVtimeBounds(taxisID2, vtime_lb, vtime_ub);
       streamDefTimestep(streamID2, otsID);
 
       if ( cdoDiag )
 	{
 	  taxisDefVdate(taxisID3, vdate0);
 	  taxisDefVtime(taxisID3, vtime0);
-	  if ( taxis_has_bounds )
-	    {
-	      taxisDefVdateBounds(taxisID3, vdate_lb, vdate_ub);
-	      taxisDefVtimeBounds(taxisID3, vtime_lb, vtime_ub);
-	    }
+	  taxisDefVdateBounds(taxisID3, vdate_lb, vdate_ub);
+	  taxisDefVtimeBounds(taxisID3, vtime_lb, vtime_ub);
 	  streamDefTimestep(streamID3, otsID);
 	}
 
