@@ -21,10 +21,6 @@
       Tstepcount  tstepcount  Count number of timesteps
 */
 
-#if defined(_OPENMP)
-#  include <omp.h>
-#endif
-
 #include <cdi.h>
 #include "cdo.h"
 #include "cdo_int.h"
@@ -57,7 +53,6 @@ double tstepcount(long nts, double missval1, double *array1, double refval)
 
 void *Tstepcount(void *argument)
 {
-  int ompthID;
   int gridsize;
   int nrecs;
   int gridID, varID, levelID, recID;
@@ -111,7 +106,7 @@ void *Tstepcount(void *argument)
       if ( tsID >= nalloc )
 	{
 	  nalloc += NALLOC_INC;
-	  vars  = realloc(vars, nalloc*sizeof(field_t **));
+	  vars  = (field_t ***) realloc(vars, nalloc*sizeof(field_t **));
 	}
 
       vdate = taxisInqVdate(taxisID1);
@@ -124,7 +119,7 @@ void *Tstepcount(void *argument)
 	  streamInqRecord(streamID1, &varID, &levelID);
 	  gridID   = vlistInqVarGrid(vlistID1, varID);
 	  gridsize = gridInqSize(gridID);
-	  vars[tsID][varID][levelID].ptr = malloc(gridsize*sizeof(double));
+	  vars[tsID][varID][levelID].ptr = (double*) malloc(gridsize*sizeof(double));
 	  streamReadRecord(streamID1, vars[tsID][varID][levelID].ptr, &nmiss);
 	  vars[tsID][varID][levelID].nmiss = nmiss;
 	}
@@ -134,10 +129,10 @@ void *Tstepcount(void *argument)
 
   nts = tsID;
 
-  mem = malloc(ompNumThreads*sizeof(memory_t));
+  mem = (memory_t*) malloc(ompNumThreads*sizeof(memory_t));
   for ( i = 0; i < ompNumThreads; i++ )
     {
-      mem[i].array1 = malloc(nts*sizeof(double));
+      mem[i].array1 = (double*) malloc(nts*sizeof(double));
     }
 
   for ( varID = 0; varID < nvars; varID++ )
@@ -149,16 +144,13 @@ void *Tstepcount(void *argument)
       for ( levelID = 0; levelID < nlevel; levelID++ )
 	{
 #if defined(_OPENMP)
-#pragma omp parallel for default(shared) private(i, ompthID, tsID) \
+#pragma omp parallel for default(shared) private(i, tsID) \
   schedule(dynamic,1)
 #endif
 	  for ( i = 0; i < gridsize; i++ )
 	    {
-#if defined(_OPENMP)
-              ompthID = omp_get_thread_num();
-#else
-              ompthID = 0;
-#endif
+	      int ompthID = cdo_omp_get_thread_num();
+
 	      for ( tsID = 0; tsID < nts; tsID++ )
 		mem[ompthID].array1[tsID] = vars[tsID][varID][levelID].ptr[i];
 
