@@ -22,6 +22,10 @@
 #include <stdbool.h>
 #include "dmemory.h"
 #include "util.h"
+#include "text.h"
+
+/* dummy use of unused parameters to silence compiler warnings */
+#define  UNUSED(x) (void)x
 
 #undef   TRUE
 #define  TRUE   1
@@ -32,16 +36,18 @@
 #define  MIN(a,b)  ((a) < (b) ? (a) : (b))
 #undef   MAX
 #define  MAX(a,b)  ((a) > (b) ? (a) : (b))
-#undef   NINT
-#define  NINT(x)   ((x) < 0 ? (int)((x)-0.5) : (int)((x)+0.5))
-#undef   NINTD
-#define  NINTD(x)   ((x) < 0 ? ((x)-0.5) : ((x)+0.5))
+
+#define  ADD_PLURAL(n)  ((n)>1 ? "s" : "")
 
 #define  UNCHANGED_RECORD  (processSelf() == 0 && cdoStreamName(0)->argv[0][0] != '-' && cdoRegulargrid == FALSE && cdoDefaultFileType == -1 && cdoDefaultDataType == -1 && cdoDefaultByteorder == -1 )
 
 
 extern int ompNumThreads;
 
+extern int stdin_is_tty;
+extern int stdout_is_tty;
+extern int stderr_is_tty;
+
 extern int cdoDefaultFileType;
 extern int cdoDefaultDataType;
 extern int cdoDefaultByteorder;
@@ -69,12 +75,17 @@ extern int cdoChunkType;
 
 extern int cdoExpMode;
 
+extern int CDO_Color;
+extern int CDO_Use_FFTW;
 extern int cdoDiag;
 
 extern int cdoNumVarnames;
 extern char **cdoVarnames;
 
-void    cdiError(int cdiErrno, const char *fmt, ...);
+int cdo_omp_get_thread_num(void);
+
+
+void    cdiOpenError(int cdiErrno, const char *fmt, const char *path);
 void    cdoAbort(const char *fmt, ...);
 void    cdoWarning(const char *fmt, ...);
 void    cdoPrint(const char *fmt, ...);
@@ -85,6 +96,7 @@ void timer_start(int it);
 void timer_stop(int it);
 double timer_val(int it);
 
+
 void    operatorInputArg(const char *enter);
 int     operatorArgc(void);
 char  **operatorArgv(void);
