@@ -3,6 +3,7 @@
 #include <string.h>
 #include <math.h>
 #include <errno.h>
+#include <ctype.h>
 
 #include <cdi.h>
 #include "cdo.h"
@@ -14,7 +15,7 @@
 
 static double f_abs(double x)  { return (fabs(x));  }
 static double f_int(double x)  { return ((int)(x)); }
-static double f_nint(double x) { return (NINT(x));  }
+static double f_nint(double x) { return (round(x));  }
 static double f_sqr(double x)  { return (x*x);      }
 
 typedef struct {
@@ -70,7 +71,7 @@ nodeType *expr_con_con(int oper, nodeType *p1, nodeType *p2)
 {
   nodeType *p;
 
-  p = malloc(sizeof(nodeType));
+  p = (nodeType*) malloc(sizeof(nodeType));
 
   p->type = typeCon;
 
@@ -106,7 +107,7 @@ nodeType *expr_con_var(int oper, nodeType *p1, nodeType *p2)
   ngp  = gridInqSize(gridID);
   nlev = zaxisInqSize(zaxisID);
 
-  p = malloc(sizeof(nodeType));
+  p = (nodeType*) malloc(sizeof(nodeType));
 
   p->type     = typeVar;
   p->tmpvar   = 1;
@@ -115,7 +116,7 @@ nodeType *expr_con_var(int oper, nodeType *p1, nodeType *p2)
   p->zaxisID  = zaxisID;
   p->missval  = missval1;
 
-  p->data = malloc(ngp*nlev*sizeof(double));
+  p->data = (double*) malloc(ngp*nlev*sizeof(double));
 
   switch ( oper )
     {
@@ -208,7 +209,7 @@ nodeType *expr_var_con(int oper, nodeType *p1, nodeType *p2)
   ngp  = gridInqSize(gridID);
   nlev = zaxisInqSize(zaxisID);
 
-  p = malloc(sizeof(nodeType));
+  p = (nodeType*) malloc(sizeof(nodeType));
 
   p->type     = typeVar;
   p->tmpvar   = 1;
@@ -217,7 +218,7 @@ nodeType *expr_var_con(int oper, nodeType *p1, nodeType *p2)
   p->zaxisID  = zaxisID;
   p->missval  = missval1;
 
-  p->data = malloc(ngp*nlev*sizeof(double));
+  p->data = (double*) malloc(ngp*nlev*sizeof(double));
 
   switch ( oper )
     {
@@ -323,7 +324,7 @@ nodeType *expr_var_var(int oper, nodeType *p1, nodeType *p2)
   nlev1 = zaxisInqSize(p1->zaxisID);
   nlev2 = zaxisInqSize(p2->zaxisID);
 
-  p = malloc(sizeof(nodeType));
+  p = (nodeType*) malloc(sizeof(nodeType));
 
   p->type     = typeVar;
   p->tmpvar   = 1;
@@ -353,7 +354,7 @@ nodeType *expr_var_var(int oper, nodeType *p1, nodeType *p2)
       p->missval = p1->missval;
     }
 
-  p->data = malloc(ngp*nlev*sizeof(double));
+  p->data = (double*) malloc(ngp*nlev*sizeof(double));
 
   for ( k = 0; k < nlev; k++ )
     {
@@ -517,7 +518,7 @@ nodeType *ex_fun_con(char *fun, nodeType *p1)
   int i;
   int funcID = -1;
 
-  p = malloc(sizeof(nodeType));
+  p = (nodeType*) malloc(sizeof(nodeType));
 
   p->type = typeCon;
 
@@ -556,7 +557,7 @@ nodeType *ex_fun_var(char *fun, nodeType *p1)
   ngp  = gridInqSize(gridID);
   nlev = zaxisInqSize(zaxisID);
 
-  p = malloc(sizeof(nodeType));
+  p = (nodeType*) malloc(sizeof(nodeType));
 
   p->type     = typeVar;
   p->tmpvar   = 1;
@@ -565,7 +566,7 @@ nodeType *ex_fun_var(char *fun, nodeType *p1)
   p->zaxisID  = zaxisID;
   p->missval  = missval;
 
-  p->data = malloc(ngp*nlev*sizeof(double));
+  p->data = (double*) malloc(ngp*nlev*sizeof(double));
 
   for ( i = 0; i < NumFunc; i++)
     if ( strcmp(fun, fun_sym_tbl[i].name) == 0 )
@@ -648,7 +649,7 @@ nodeType *ex_uminus_var(nodeType *p1)
   ngp  = gridInqSize(gridID);
   nlev = zaxisInqSize(zaxisID);
 
-  p = malloc(sizeof(nodeType));
+  p = (nodeType*) malloc(sizeof(nodeType));
 
   p->type     = typeVar;
   p->tmpvar   = 1;
@@ -657,7 +658,7 @@ nodeType *ex_uminus_var(nodeType *p1)
   p->zaxisID  = zaxisID;
   p->missval  = missval;
 
-  p->data = malloc(ngp*nlev*sizeof(double));
+  p->data = (double*) malloc(ngp*nlev*sizeof(double));
 
   if ( nmiss > 0 )
     {
@@ -680,7 +681,7 @@ nodeType *ex_uminus_con(nodeType *p1)
 {
   nodeType *p;
 
-  p = malloc(sizeof(nodeType));
+  p = (nodeType*) malloc(sizeof(nodeType));
 
   p->type = typeCon;
 
@@ -862,8 +863,17 @@ nodeType *expr_run(nodeType *p, parse_parm_t *parse_arg)
 		cdoAbort("Operand not variable!");
 
 	      varID = vlistDefVar(parse_arg->vlistID2, parse_arg->gridID2, parse_arg->zaxisID2, parse_arg->tsteptype2);
-	      vlistDefVarName(parse_arg->vlistID2, varID, p->u.opr.op[0]->u.var.nm);
+	      const char *varname = p->u.opr.op[0]->u.var.nm;
+	      vlistDefVarName(parse_arg->vlistID2, varID, varname);
 	      vlistDefVarMissval(parse_arg->vlistID2, varID, parse_arg->missval2);
+	      if ( memcmp(varname, "var", 3) == 0 )
+		{
+		  if ( strlen(varname) > 3 && isdigit(varname[3]) )
+		    {
+		      int code = atoi(varname+3);
+		      vlistDefVarCode(parse_arg->vlistID2, varID, code);
+		    }
+		}
 	    }
 	  else
 	    {
