@@ -55,7 +55,7 @@ enum cmp_flag {
   CMP_GRIDSIZE = 16,
   CMP_HRD      = CMP_CODE | CMP_GRIDSIZE,
   CMP_DIM      =            CMP_GRIDSIZE | CMP_NLEVEL | CMP_GRID,
-  CMP_ALL      = CMP_NAME | CMP_GRIDSIZE | CMP_NLEVEL | CMP_GRID,
+  CMP_ALL      = CMP_NAME | CMP_GRIDSIZE | CMP_NLEVEL | CMP_GRID
 };
 
 void    vlistCompare(int vlistID1, int vlistID2, int flag);
