@@ -55,11 +55,11 @@ void scale_vec(double scalefactor, long nvals, double *restrict values)
 
 void grid_to_radian(const char *units, long nvals, double *restrict values, const char *description)
 {
-  if ( memcmp(units, "degree", 6) == 0 )
+  if ( strcompare(units, "degree") == 0 )
     {
       scale_vec(DEG2RAD, nvals, values);
     }
-  else if ( memcmp(units, "radian", 6) == 0 )
+  else if ( strcompare(units, "radian") == 0 )
     {
       /* No conversion necessary */
     }
@@ -72,11 +72,11 @@ void grid_to_radian(const char *units, long nvals, double *restrict values, cons
 
 void grid_to_degree(const char *units, long nvals, double *restrict values, const char *description)
 {
-  if ( memcmp(units, "radian", 6) == 0 )
+  if ( strcompare(units, "radian") == 0 )
     {
       for ( long n = 0; n < nvals; ++n ) values[n] *= RAD2DEG;
     }
-  else if ( memcmp(units, "degree", 6) == 0 )
+  else if ( strcompare(units, "degree") == 0 )
     {
       /* No conversion necessary */
     }
@@ -109,7 +109,7 @@ int gridToZonal(int gridID1)
 
       if ( gridInqYvals(gridID1, NULL) )
 	{
-	  yvals = malloc(gridsize*sizeof(double));
+	  yvals = (double*) malloc(gridsize*sizeof(double));
 
 	  gridInqYvals(gridID1, yvals);
 	  gridDefYvals(gridID2, yvals);
@@ -146,7 +146,7 @@ int gridToMeridional(int gridID1)
 
       if ( gridInqXvals(gridID1, NULL) )
 	{
-	  xvals = malloc(gridsize*sizeof(double));
+	  xvals = (double*) malloc(gridsize*sizeof(double));
 
 	  gridInqXvals(gridID1, xvals);
 	  gridDefXvals(gridID2, xvals);
@@ -169,13 +169,19 @@ void grid_gen_corners(long n, const double* restrict vals, double* restrict corn
 {
   long i;
 
-  for ( i = 0; i < n-1; ++i )
+  if ( n == 1 )
     {
-      corners[i+1] = 0.5*(vals[i] + vals[i+1]);
+      corners[0] = vals[0];
+      corners[1] = vals[0];
     }
+  else
+    {
+      for ( i = 0; i < n-1; ++i )
+	corners[i+1] = 0.5*(vals[i] + vals[i+1]);
 
-  corners[0] = 2*vals[0] - corners[1];
-  corners[n] = 2*vals[n-1] - corners[n-1];
+      corners[0] = 2*vals[0] - corners[1];
+      corners[n] = 2*vals[n-1] - corners[n-1];
+    }
 }
 
 
@@ -451,7 +457,7 @@ char *gen_param(const char *fmt, ...)
   va_end(args);
 
   len++;
-  rstr = malloc(len*sizeof(char));
+  rstr = (char*) malloc(len*sizeof(char));
   memcpy(rstr, str, len*sizeof(char));
 
   return (rstr);
@@ -464,7 +470,6 @@ void lcc_to_geo(int gridID, int gridsize, double *xvals, double *yvals)
   double zlat, zlon;
   double xi, xj;
   int projflag, scanflag;
-  int status;
   long i;
   proj_info_t proj;
 
@@ -692,8 +697,8 @@ void grib_get_reduced_row(long pl,double lon_first,double lon_last,long* npoints
 }
 
 
-int    qu2reg3(double *pfield, int *kpoint, int klat, int klon,
-	       double msval, int *kret, int omisng, int operio, int oveggy);
+int qu2reg3_double(double *pfield, int *kpoint, int klat, int klon,
+		   double msval, int *kret, int omisng, int operio, int oveggy);
 
 static
 int qu2reg_subarea(int gridsize, int np, double xfirst, double xlast, 
@@ -721,8 +726,8 @@ int qu2reg_subarea(int gridsize, int np, double xfirst, double xlast,
 
   for ( j = 0; j < ny; ++j ) nwork += rowlon[j];
 
-  pwork = malloc(ny*sizeof(double *));
-  work  = malloc(ny*np4*sizeof(double));
+  pwork = (double **) malloc(ny*sizeof(double *));
+  work  = (double*) malloc(ny*np4*sizeof(double));
   wlen = 0;
   pwork[0] = work;
   for ( j = 1; j < ny; ++j )
@@ -758,7 +763,7 @@ int qu2reg_subarea(int gridsize, int np, double xfirst, double xlast,
 
   if ( gridsize != size ) cdoAbort("gridsize1 inconsistent!");
 
-  (void) qu2reg3(work, rowlon, ny, np4, missval, iret, lmiss, lperio, lveggy);
+  (void) qu2reg3_double(work, rowlon, ny, np4, missval, iret, lmiss, lperio, lveggy);
 
   wlen = 0;
   pwork[0] = work;
@@ -809,7 +814,7 @@ void field2regular(int gridID1, int gridID2, double missval, double *array, int
   ny = gridInqYsize(gridID1);
   np = gridInqNP(gridID1);
 
-  rowlon = malloc(ny*sizeof(int));
+  rowlon = (int*) malloc(ny*sizeof(int));
   gridInqRowlon(gridID1, rowlon);
 
   xfirstandlast[0] = 0.;
@@ -825,7 +830,7 @@ void field2regular(int gridID1, int gridID2, double missval, double *array, int
   else
     {
       nx = 2*ny;
-      (void) qu2reg3(array, rowlon, ny, nx, missval, &iret, lmiss, lperio, lveggy);
+      (void) qu2reg3_double(array, rowlon, ny, nx, missval, &iret, lmiss, lperio, lveggy);
     }
 
   if ( gridInqSize(gridID2) != nx*ny ) Error("Gridsize differ!");
@@ -851,7 +856,7 @@ int gridToRegular(int gridID1)
   ny = gridInqYsize(gridID1);
   np = gridInqNP(gridID1);
 
-  yvals = malloc(ny*sizeof(double));
+  yvals = (double*) malloc(ny*sizeof(double));
   gridInqYvals(gridID1, yvals);
 
   xfirstandlast[0] = 0.;
@@ -873,7 +878,7 @@ int gridToRegular(int gridID1)
       grib_get_reduced_row(np4, xfirst, xlast, &row_count, &ilon_first, &ilon_last);
 
       nx = row_count;
-      xvals = malloc(nx*sizeof(double));
+      xvals = (double*) malloc(nx*sizeof(double));
       for ( i = 0; i < nx; ++i )
 	{
 	  xvals[i] = ((ilon_first+i)*360.)/np4;
@@ -885,7 +890,7 @@ int gridToRegular(int gridID1)
   else
     {
       nx = 2*ny;
-      xvals = malloc(nx*sizeof(double));
+      xvals = (double*) malloc(nx*sizeof(double));
       for ( i = 0; i < nx; ++i ) xvals[i] = i * 360./nx;
     }
 
@@ -912,7 +917,7 @@ void gridCopyMask(int gridID1, int gridID2, long gridsize)
   if ( gridInqMask(gridID1, NULL) )
     {
       int *mask;
-      mask = malloc(gridsize*sizeof(int));
+      mask = (int*) malloc(gridsize*sizeof(int));
       gridInqMask(gridID1, mask);
       gridDefMask(gridID2, mask);
       free(mask);
@@ -997,8 +1002,8 @@ int gridToCurvilinear(int gridID1, int lbounds)
 	gridDefXsize(gridID2, nx);
 	gridDefYsize(gridID2, ny);
 
-	xvals2D = malloc(gridsize*sizeof(double));
-	yvals2D = malloc(gridsize*sizeof(double));
+	xvals2D = (double*) malloc(gridsize*sizeof(double));
+	yvals2D = (double*) malloc(gridsize*sizeof(double));
 
 
 	if ( gridtype == GRID_LCC )
@@ -1017,8 +1022,8 @@ int gridToCurvilinear(int gridID1, int lbounds)
 	    if ( ! (gridInqXvals(gridID1, NULL) && gridInqYvals(gridID1, NULL)) )
 	      Error("Grid has no values");
 
-	    xvals = malloc(nx*sizeof(double));
-	    yvals = malloc(ny*sizeof(double));
+	    xvals = (double*) malloc(nx*sizeof(double));
+	    yvals = (double*) malloc(ny*sizeof(double));
 
 	    gridInqXvals(gridID1, xvals);
 	    gridInqYvals(gridID1, yvals);
@@ -1073,8 +1078,8 @@ int gridToCurvilinear(int gridID1, int lbounds)
 
 	if ( gridtype == GRID_LCC )
 	  {		
-	    xbounds2D = malloc(4*gridsize*sizeof(double));
-	    ybounds2D = malloc(4*gridsize*sizeof(double));
+	    xbounds2D = (double*) malloc(4*gridsize*sizeof(double));
+	    ybounds2D = (double*) malloc(4*gridsize*sizeof(double));
 
 	    for ( j = 0; j < ny; j++ )
 	      for ( i = 0; i < nx; i++ )
@@ -1106,7 +1111,7 @@ int gridToCurvilinear(int gridID1, int lbounds)
 	  {
 	    if ( gridInqXbounds(gridID1, NULL) )
 	      {
-		xbounds = malloc(2*nx*sizeof(double));
+		xbounds = (double*) malloc(2*nx*sizeof(double));
 		gridInqXbounds(gridID1, xbounds);
 		if ( gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN )
 		  if ( check_range(2*nx, xbounds, -720, 720) )
@@ -1118,13 +1123,13 @@ int gridToCurvilinear(int gridID1, int lbounds)
 	      }
 	    else if ( nx > 1 )
 	      {
-		xbounds = malloc(2*nx*sizeof(double));
+		xbounds = (double*) malloc(2*nx*sizeof(double));
 		grid_gen_bounds(nx, xvals, xbounds);
 	      }
 
 	    if ( gridInqYbounds(gridID1, NULL) )
 	      {
-		ybounds = malloc(2*ny*sizeof(double));
+		ybounds = (double*) malloc(2*ny*sizeof(double));
 		gridInqYbounds(gridID1, ybounds);
 		if ( gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN )
 		  if ( check_range(2*ny, ybounds, -180, 180) )
@@ -1136,7 +1141,7 @@ int gridToCurvilinear(int gridID1, int lbounds)
 	      }
 	    else if ( ny > 1 )
 	      {
-		ybounds = malloc(2*ny*sizeof(double));
+		ybounds = (double*) malloc(2*ny*sizeof(double));
 		if ( gridtype == GRID_SINUSOIDAL || 
 		     gridtype == GRID_LAEA       || 
 		     gridtype == GRID_LCC2 )
@@ -1150,8 +1155,8 @@ int gridToCurvilinear(int gridID1, int lbounds)
 
 	    if ( xbounds && ybounds )
 	      {
-		xbounds2D = malloc(4*gridsize*sizeof(double));
-		ybounds2D = malloc(4*gridsize*sizeof(double));
+		xbounds2D = (double*) malloc(4*gridsize*sizeof(double));
+		ybounds2D = (double*) malloc(4*gridsize*sizeof(double));
 
 		if ( gridIsRotated(gridID1) )
 		  {
@@ -1185,7 +1190,7 @@ int gridToCurvilinear(int gridID1, int lbounds)
 			  {
 			    sinusoidal_to_geo(4*gridsize, xbounds2D, ybounds2D);
 			    /*
-			    xvals2D = malloc(gridsize*sizeof(double));
+			    xvals2D = (double*) malloc(gridsize*sizeof(double));
 			    for ( j = 0; j < 4; ++j )
 			      {
 				for ( i = 0; i < gridsize; ++i ) xvals2D[i] = xbounds2D[i*4+j];
@@ -1272,11 +1277,11 @@ int gridToUnstructured(int gridID1, int lbounds)
 	gridDefXsize(gridID2, gridsize);
 	gridDefYsize(gridID2, gridsize);
 
-	xvals = malloc(nx*sizeof(double));
-	yvals = malloc(ny*sizeof(double));
+	xvals = (double*) malloc(nx*sizeof(double));
+	yvals = (double*) malloc(ny*sizeof(double));
 
-	xvals2D = malloc(gridsize*sizeof(double));
-	yvals2D = malloc(gridsize*sizeof(double));
+	xvals2D = (double*) malloc(gridsize*sizeof(double));
+	yvals2D = (double*) malloc(gridsize*sizeof(double));
 
 	gridInqXvals(gridID1, xvals);
 	gridInqYvals(gridID1, yvals);
@@ -1319,31 +1324,31 @@ int gridToUnstructured(int gridID1, int lbounds)
 
 	    if ( gridInqXbounds(gridID1, NULL) )
 	      {
-		xbounds = malloc(2*nx*sizeof(double));
+		xbounds = (double*) malloc(2*nx*sizeof(double));
 		gridInqXbounds(gridID1, xbounds);
 	      }
 	    else if ( nx > 1 )
 	      {
-		xbounds = malloc(2*nx*sizeof(double));
+		xbounds = (double*) malloc(2*nx*sizeof(double));
 		grid_gen_bounds(nx, xvals, xbounds);
 	      }
 
 	    if ( gridInqYbounds(gridID1, NULL) )
 	      {
-		ybounds = malloc(2*ny*sizeof(double));
+		ybounds = (double*) malloc(2*ny*sizeof(double));
 		gridInqYbounds(gridID1, ybounds);
 	      }
 	    else if ( ny > 1 )
 	      {
-		ybounds = malloc(2*ny*sizeof(double));
+		ybounds = (double*) malloc(2*ny*sizeof(double));
 		grid_gen_bounds(ny, yvals, ybounds);
 		grid_check_lat_borders(2*ny, ybounds);
 	      }
 
 	    if ( xbounds && ybounds )
 	      {
-		xbounds2D = malloc(4*gridsize*sizeof(double));
-		ybounds2D = malloc(4*gridsize*sizeof(double));
+		xbounds2D = (double*) malloc(4*gridsize*sizeof(double));
+		ybounds2D = (double*) malloc(4*gridsize*sizeof(double));
 
 		if ( gridIsRotated(gridID1) )
 		  {
@@ -1395,13 +1400,13 @@ int gridToUnstructured(int gridID1, int lbounds)
 	ni2 = gridInqGMEni2(gridID1);
 	ni3 = gridInqGMEni3(gridID1);
 
-	imask   = malloc(gridsize*sizeof(int));
-	xvals   = malloc(gridsize*sizeof(double));
-	yvals   = malloc(gridsize*sizeof(double));
+	imask   = (int*) malloc(gridsize*sizeof(int));
+	xvals   = (double*) malloc(gridsize*sizeof(double));
+	yvals   = (double*) malloc(gridsize*sizeof(double));
 	if ( lbounds )
 	  {
-	    xbounds = malloc(nv*gridsize*sizeof(double));
-	    ybounds = malloc(nv*gridsize*sizeof(double));
+	    xbounds = (double*) malloc(nv*gridsize*sizeof(double));
+	    ybounds = (double*) malloc(nv*gridsize*sizeof(double));
 	  }
 
 	gme_grid(lbounds, gridsize, xvals, yvals, xbounds, ybounds, imask, ni, nd, ni2, ni3);
@@ -1464,7 +1469,6 @@ int gridCurvilinearToRegular(int gridID1)
 {
   int gridID2 = -1;
   int gridtype, gridsize;
-  long index;
   long i, j;
   int nx, ny;
   int lx = TRUE, ly = TRUE;
@@ -1479,14 +1483,14 @@ int gridCurvilinearToRegular(int gridID1)
   nx = gridInqXsize(gridID1);
   ny = gridInqYsize(gridID1);
 	
-  xvals2D = malloc(gridsize*sizeof(double));
-  yvals2D = malloc(gridsize*sizeof(double));
+  xvals2D = (double*) malloc(gridsize*sizeof(double));
+  yvals2D = (double*) malloc(gridsize*sizeof(double));
 
   gridInqXvals(gridID1, xvals2D);
   gridInqYvals(gridID1, yvals2D);
 
-  xvals = malloc(nx*sizeof(double));
-  yvals = malloc(ny*sizeof(double));
+  xvals = (double*) malloc(nx*sizeof(double));
+  yvals = (double*) malloc(ny*sizeof(double));
 
   for ( i = 0; i < nx; i++ ) xvals[i] = xvals2D[i];
   for ( j = 0; j < ny; j++ ) yvals[j] = yvals2D[j*nx];
@@ -1556,7 +1560,7 @@ int gridGenWeights(int gridID, double *grid_area, double *grid_wgts)
   if ( gridtype == GRID_GME )
     {
       gridID = gridToUnstructured(gridID, 1);	  
-      grid_mask = malloc(gridsize*sizeof(int));
+      grid_mask = (int*) malloc(gridsize*sizeof(int));
       gridInqMaskGME(gridID, grid_mask);
     }
 
@@ -1686,7 +1690,7 @@ int gridWeights(int gridID, double *grid_wgts)
   gridtype = gridInqType(gridID);
   gridsize = gridInqSize(gridID);
   
-  grid_area = malloc(gridsize*sizeof(double));
+  grid_area = (double*) malloc(gridsize*sizeof(double));
 
   a_status = 0;
 
