@@ -228,39 +228,36 @@ int referenceToGrid(int gridID1)
 	  position = gridInqPosition(gridID1);
 
 	  streamID = streamOpenRead(gridpath);
-	  if ( streamID >= 0 )
+	  if ( streamID < 0 ) cdiOpenError(streamID, "Open failed on horizontal grid file >%s<", gridpath);
+
+	  int vlistID, gridID = -1;
+	  int ngrids;
+	  vlistID = streamInqVlist(streamID);
+	  ngrids = vlistNgrids(vlistID);
+	  if ( position > 0 && position <= ngrids )
 	    {
-	      int vlistID, gridID = -1;
-	      int ngrids;
-	      vlistID = streamInqVlist(streamID);
-	      ngrids = vlistNgrids(vlistID);
-	      if ( position > 0 && position <= ngrids )
-		{
-		  gridID = vlistGrid(vlistID, position-1);
-		  if ( gridInqSize(gridID) == gridsize )
-		    gridID2 = gridDuplicate(gridID);
-		  else
-		    cdoWarning("Grid size %d on position %d do not match! Reference=%s", gridsize, position, gridpath);
-		}
-	      else if ( position == 0 )
-		{
-		  for ( int grididx = 0; grididx < ngrids; ++grididx )
-		    {
-		      gridID = vlistGrid(vlistID, grididx);
-		      if ( gridInqSize(gridID) == gridsize )
-			{
-			  gridID2 = gridDuplicate(gridID);
-			  break;
-			}
-		    }
-		}
+	      gridID = vlistGrid(vlistID, position-1);
+	      if ( gridInqSize(gridID) == gridsize )
+		gridID2 = gridDuplicate(gridID);
 	      else
-		cdoWarning("Number of grid in reference %d not available! Reference=%s", position, gridpath);
-	      
-	      streamClose(streamID);
+		cdoWarning("Grid size %d on position %d do not match! Reference=%s", gridsize, position, gridpath);
+	    }
+	  else if ( position == 0 )
+	    {
+	      for ( int grididx = 0; grididx < ngrids; ++grididx )
+		{
+		  gridID = vlistGrid(vlistID, grididx);
+		  if ( gridInqSize(gridID) == gridsize )
+		    {
+		      gridID2 = gridDuplicate(gridID);
+		      break;
+			}
+		}
 	    }
 	  else
-	    cdiError(streamID, "Open failed on horizontal grid file >%s<", gridpath);
+	    cdoWarning("Number of grid in reference %d not available! Reference=%s", position, gridpath);
+	  
+	  streamClose(streamID);
 	}
 
       if ( gridID2 != -1 )
